/*
 *************************************************************************
 *                                                                       *
 * Copyright 2000 Compaq Computer Corporation                            *
 *                                                                       *
 * COMPAQ Registered in U.S. Patent and Trademark Office.                *
 *                                                                       *
 *************************************************************************
 * IMPORTANT: Carefully read the License Terms below before              *
 * proceeding.  By use of these materials you agree to these terms.      *
 * If you do not agree to these terms, you may not use this software or  *
 * the accompanying documentation.                                       *
 *************************************************************************
 * LICENSE TERMS                                                         *
 * 1. GRANT                                                              *
 * Compaq Computer Corporation ("COMPAQ") grants you the right to use    *
 * and modify the following source code (the "Software") for use on any  *
 * number of computers.                                                  *
 * 2. COPYRIGHT                                                          *
 * The Software is owned by COMPAQ and its suppliers and is protected by *
 * copyright laws and international treaties.  Your use of the Software  *
 * and associated documentation is subject to the applicable copyright   *
 * laws and the express rights and restrictions of these terms.          *
 * 3. RESTRICTIONS                                                       *
 * You may not sell, rent, lease, license or otherwise commercially      *
 * transfer the Software.  You may not remove any copyright, trademark,  *
 * or other proprietary notices from the Software or the associated      *
 * documentation.   You are responsible for compliance with all          *
 * applicable export or re-export control laws and regulations if you    *
 * export the Software. This license is governed by and is to be         *
 * construed under the laws of the State of Texas.                       *
 *                                                                       *
 * DISCLAIMER OF WARRANTY AND LIABILITY                                  *
 * Compaq shall not be liable for technical or editorial errors or       *
 * omissions contained herein. The information contained herein is       *
 * subject to change without notice.                                     *
 *                                                                       *
 * THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND.       *
 * THE ENTIRE RISK ARISING OUT OF THE USE OF THIS SOFTWARE REMAINS WITH  *
 * RECIPIENT.  IN NO EVENT SHALL COMPAQ BE LIABLE FOR ANY DIRECT,        *
 * CONSEQUENTIAL, INCIDENTAL, SPECIAL, PUNITIVE OR OTHER DAMAGES         *
 * WHATSOEVER (INCLUDING WITHOUT LIMITATION DAMAGES FOR LOSS OF BUSINESS *
 * PROFITS, BUSINESS INTERRUPTION, OR LOSS OF BUSINESS INFORMATION),     *
 * EVEN IF COMPAQ HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES    *
 * AND WHETHER IN AN ACTION OF CONTRACT OR TORT INCLUDING NEGLIGENCE.    *
 *                                                                       *
 * If you have any questions concerning this license, please contact:    *
 * Compaq Computer Corporation, Software Business Practices, ZKO1-2/D22, *
 * 110 Spit Brook Road, Nashua, NH. 03062-2698.                          *
 *                                                                       *
 *************************************************************************
 */


#include "vms_jackets.h"
#include <limits.h>
#include <stdio.h>
#include <stat.h>
#include <time.h>
#include <file.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <unixio.h>

/*
** There are actually two stat jackets in here. The first is the normal
** file parsing one, and the second is one that handles "/" and "/DDcu".
** The first calls the second so both are in effect.
*/

/*
** Debugging/tracing macros.
*/
#ifdef DEBUG
const char debug_stat[]=VMS_PREFIX "DEBUG_STAT";
#define DPRINT(s) if (getenv(debug_stat)) printf(s)
#define DPRINT_1(s,p1) if (getenv(debug_stat)) printf(s,p1)
#define DPRINT_2(s,p1,p2) if (getenv(debug_stat)) printf(s,p1,p2)
#define DPRINT_3(s,p1,p2,p3) if (getenv(debug_stat)) printf(s,p1,p2,p3)
#define DPRINT_4(s,p1,p2,p3,p4) if (getenv(debug_stat)) printf(s,p1,p2,p3,p4)
#else
#define DPRINT(s)
#define DPRINT_1(s,p1)
#define DPRINT_2(s,p1,p2)
#define DPRINT_3(s,p1,p2,p3)
#define DPRINT_4(s,p1,p2,p3,p4)
#endif

/*
** This is the template for the root stat structure. Of course, such a
** thing doesn't exist on OpenVMS, so this is pure fabrication.
**
** Notice that because we need to be able to return a stat struct for our
** fabricated root, we need to tie its fileid to some unique device name.
** We could use anything, and it probably doesn't need to even be a valid
** device, but we use the NULL device. Hey, its about time someone used
** the poor null device.
*/
char stat_root_dev[] = "_NLA0";
struct stat stat_root =
{
  NULL,                 /* st_dev - pointer to physical device name */
  4, 4, 0,              /* st_ino[3] - 3 words to receive fid */
  S_IFDIR,              /* st_mode - file "mode" i.e. prot, dir, reg, etc.
  0,                    /* st_nlink - for compatibility - not really used */
  0x10001,              /* st_uid - from ACP - QIO uic field */
  0x1,                  /* st_gid - group number extracted from st_uid */
  0,                    /* st_rdev - for compatibility - always zero */
  0,                    /* st_size - file size in bytes */
  0,                    /* st_atime - file access time; always same as st_mtime
  0,                    /* st_mtime - last modification time */
  0,                    /* st_ctime - file creation time */
  0,                    /* st_fab_rfm - record format - UDF */
  0,                    /* st_fab_rat - record attributes - none */
  0,                    /* st_fab_fsz - fixed header size */
  0                     /* st_fab_mrs - record size */
};

int GENERIC_STAT_JACKET2(const char *path, struct stat *buf) {

    int rv, len;
    char my_path[PATH_MAX+8];

    /*
    ** Check that we have the room to do our special stuff.
    */
    len=strlen(path);
    if (len <= PATH_MAX) {

        /*
        ** First off, tidy the spec up a little.
        */
        generic_tidy_spec(my_path, path);
        DPRINT_2("GENERIC_stat_Jacket: %s, tidied %s\n",path,my_path);

        /*
        ** If its the root directory, then we return the fake stat block. We
        ** update the access and modified times so the caller doesn't maintain
        ** any potentially out-of-date information.
        */
        if ( (strlen(my_path) == 1) && (my_path[0] == '/') ) {
            memmove(buf, &stat_root, sizeof(stat_root));
            buf->st_dev = stat_root_dev;
            buf->st_atime = buf->st_mtime = time(NULL);
            DPRINT("GENERIC_stat_Jacket: Was root\n");
            DPRINT_4("GENERIC_stat_Jacket: (a) %s %d,%d,%d\n",buf->st_dev,
                buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
            return 0;
        }

        /*
        ** If its an absolute spec with only one term, then try appending
        ** 000000 and see if that works. We have to do this first otherwise
        ** VMS's concept of the default directory can get in the way if
        ** we're dealing with a device name and that device is where the
        ** current default directory is. Eg. if the current default
        ** directory is DKA0:[X.Y] then a stat of "/dka0" will actually
        ** return informatino about "/dka0/x/y".
        */
        if (generic_absone(my_path)) {
            char copy[PATH_MAX+7+1];
            strcpy(copy,my_path);
            len = strlen(copy);
            if (copy[len-1] == '/')
                copy[len-1] = '\0';
            strcat(copy,"/000000");
            rv=stat(copy,buf);
            DPRINT_2("GENERIC_stat_Jacket: absone true, stat(%s)=%d\n",copy,rv);
            if (rv == 0) {
                DPRINT_4("GENERIC_stat_Jacket: (b) %s %d,%d,%d\n",buf->st_dev,
                    buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
                return 0;
            }
        }
    }

    /*
    ** Nothing special, so do a regular stat, but on the tidied spec.
    */
    DPRINT_1("GENERIC_stat_Jacket: calling normal stat with %s\n",my_path);
    rv=stat(my_path,buf);
    if (rv == 0) {
        DPRINT_4("GENERIC_stat_Jacket: (c) %s %d,%d,%d\n",buf->st_dev,
            buf->st_ino[0], buf->st_ino[1], buf->st_ino[2] );
        return 0;
    }
    else {
        DPRINT_1("GENERIC_stat_Jacket: (c) rv was %d\n",rv);
        return rv;
    }
}




/**********************************************************************/
/*  Jacket routines to handle VMS and decc limits                     */
/*                                                                    */
/*  Routine: GENERIC_STAT_JACKET                                      */
/*                                                                    */
/*  Jacket for: stat                                                  */
/*                                                                    */
/**********************************************************************/
int GENERIC_STAT_JACKET(const char *file_spec,struct stat *__buf)
{
   char temp_string[TEMP_STRING_SIZE];
   char temp_dir_string[TEMP_STRING_SIZE];
   char *new_filespec;
   int temp_status,status;

   new_filespec = GENERIC_VMS_Filename_parser((char *)file_spec,
                                              temp_string,
					      GENERIC_K_STRING_TYPE_UNKNOWN,
					      &temp_status);
   GENERIC_TRACE("stat_Jacket: Mapped:", file_spec, "To:", new_filespec);

   status = GENERIC_STAT_JACKET2(new_filespec,__buf);
   if (!(status==0)) /* could be a directory with several .'s check it out*/
	{
	   new_filespec = GENERIC_VMS_Filename_parser((char *)file_spec,
                                              temp_dir_string,
					      GENERIC_K_STRING_TYPE_DIRECTORY,
					      &temp_status);
	   if (strcmp(temp_string,temp_dir_string))/* diff stat it now*/
		status = stat(new_filespec,__buf);
	
	}
   return(status);
}

/*
** This is messy, but what can we do.
**
** stat.h prototypes a routine named stat but does so with a __UTC prefix
** pragma. Since we have stat defined to be GENERIC_STAT_JACKET the
** compiler/linker between themselves end up converting a stat call to 
** a call to __UTC_GENERIC_STAT_JACKET. The simplest way to fix this
** is to give it such a routine, that does nothing except call our jacket.
*/
int __UTC_GENERIC_STAT_JACKET(const char *file_spec,struct stat *__buf) {
    return GENERIC_STAT_JACKET(file_spec,__buf);
}

